/*
 * This file is part of essential.
 *
 *     essential is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     essential is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with essential.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.craftforge.essential.client;

import net.craftforge.essential.core.constants.Charsets;
import net.craftforge.essential.core.constants.HttpHeaders;
import net.craftforge.essential.core.constants.MediaTypes;
import net.craftforge.essential.core.resolvers.*;
import net.craftforge.essential.core.utils.HeaderUtils;
import net.craftforge.essential.supply.Consumer;
import net.craftforge.essential.supply.designators.ConsumerDesignator;
import net.craftforge.essential.supply.exceptions.BadInputException;
import net.craftforge.essential.supply.exceptions.UnsupportedMediaTypeException;

import java.io.InputStream;
import java.io.UnsupportedEncodingException;

/**
 * A client response is used to provide a convenient and normalized response
 * output for a client.
 *
 * @author Christian Bick
 * @since 04.07.11
 */
public class ClientResponse {

    private int statusCode;
    private HeaderResolver headerResolver;
    private BodyResolver bodyResolver;

    /**
     * Creates a client response using the given HTTP status code,
     * response headers and response body.
     *
     * @param statusCode The HTTP status code
     * @param headers The HTTP response headers
     * @param bodyInputStream The HTTP response body input stream
     */
    public ClientResponse(int statusCode, HeaderMap headers, InputStream bodyInputStream) {
        this.statusCode = statusCode;
        this.headerResolver = new DefaultHeaderResolver(headers);
        this.bodyResolver = new DefaultBodyResolver(bodyInputStream);
    }

    /**
     * Gets the HTTP status code.
     *
     * @return The HTTP status code
     */
    public int getStatusCode() {
        return statusCode;
    }

    /**
     * Gets the HTTP response body input stream.
     *
     * @return The HTTP response body input stream
     */
    public InputStream getBodyInputStream() {
        return bodyResolver.getBodyInputStream();
    }

    /**
     * Gets the HTTP response body string. Uses the charset
     * from in the Content-Type header for decoding if specified there.
     * Assumes UTF-8 otherwise.
     *
     * @return The response body string
     * @throws UnsupportedEncodingException if the request content's charset encoding is not supported
     * @throws BadInputException if the request failed because of wrong or missing request data
     */
    public String getBodyContent() throws BadInputException, UnsupportedEncodingException {
        String charset = getContentCharset();
        return bodyResolver.getBody(charset);
    }

    /**
     * Gets the HTTP response body as an object of the given class. Uses the media
     * type from the Content-Type header for deligating the deserialization to
     * right consumer method. Uses the charset from the Content-Type header for
     * decoding.
     * Assumes text/xml as media type and UTF-8 as charset if
     * not specified in the Content-Type header.
     *
     * @param clazz The class
     * @return The response body object or null if the content length is not given or 0
     * @throws UnsupportedEncodingException if the request content's charset encoding is not supported
     * @throws UnsupportedMediaTypeException if the request content's media type is not supported
     * @throws BadInputException if the request failed because of wrong or missing request data
     */
    @SuppressWarnings("unchecked")
    public <T> T getBodyObject(Class<T> clazz) throws BadInputException, UnsupportedEncodingException, UnsupportedMediaTypeException {
        int contentLength = getContentLength();
        if (contentLength == 0) {
            return null;
        }
        String mediaType = getContentMediaType();
        String charset = getContentCharset();

        ConsumerDesignator consumerDesignator = ConsumerDesignator.getInstance();
        Consumer consumer = consumerDesignator.designate(new String[]{mediaType});
        return (T) bodyResolver.getBodyAsObject(consumer, clazz, charset);
    }

    /**
     * Gets the response body's content length from the
     * Content-Length header. Assumes the content length to
     * be 0 if it is not specified in the header.
     *
     * @return The content length transmitted int the response's Content-Length header
     */
    public int getContentLength() {
        String headerValue = headerResolver.getHeaderValue(HttpHeaders.CONTENT_LENGTH, "0");
        return Integer.valueOf(headerValue);
    }

    /**
     * Gets the response body's media type from the
     * Content-Type header. Assumes the media type to
     * be text/xml if it is not specified in the header.
     *
     * @return The response body's media type
     */
    public String getContentMediaType() {
        String contentTypeHeader = headerResolver.getHeaderValue(HttpHeaders.CONTENT_TYPE, MediaTypes.TEXT_PLAIN);
        return HeaderUtils.eliminateHeaderValueAttributes(contentTypeHeader);
    }

    /**
     * Gets the response body's charset encoding from the
     * Content-Type header. Assumes the charset encoding
     * to be UTF-8 if it is not specified in the header.
     *
     * @return The response body's charset encoding
     */
    public String getContentCharset() {
        return headerResolver.getHeaderValueAttribute(HttpHeaders.CONTENT_TYPE, "charset", Charsets.UTF8);
    }
}
